@@ -1,12 +1,18 @@
-from typing import Tuple
+from __future__ import annotations
+
+import logging
+from typing import Tuple, cast
 
 from .exceptions import InverterError
 from .inverter import Inverter
 from .inverter import OperationMode
 from .inverter import SensorKind as Kind
-from .protocol import ProtocolCommand, Aa55ProtocolCommand
+from .protocol import ProtocolCommand, Aa55ProtocolCommand, Aa55ReadCommand, Aa55WriteCommand, Aa55WriteMultiCommand, \
+    ModbusReadCommand, ModbusWriteCommand, ModbusWriteMultiCommand
 from .sensor import *
 
+logger = logging.getLogger(__name__)
+
 
 class ES(Inverter):
     """Class representing inverter of ES/EM/BP family"""
@@ -55,7 +61,7 @@ class ES(Inverter):
         # Byte("cbattery3", 28, "Battery State of Charge 3", "%", Kind.BAT),
         Byte("battery_soh", 29, "Battery State of Health", "%", Kind.BAT),
         Byte("battery_mode", 30, "Battery Mode code", "", Kind.BAT),
-        Enum("battery_mode_label", 30, BATTERY_MODES_ET, "Battery Mode", Kind.BAT),
+        Enum("battery_mode_label", 30, BATTERY_MODES, "Battery Mode", Kind.BAT),
         Integer("battery_warning", 31, "Battery Warning", "", Kind.BAT),
         Byte("meter_status", 33, "Meter Status code", "", Kind.AC),
         Voltage("vgrid", 34, "On-grid Voltage", Kind.AC),
@@ -119,9 +125,7 @@ class ES(Inverter):
                    "House Consumption", "W", Kind.AC),
     )
 
-    __settings: Tuple[Sensor, ...] = (
-        EcoModeEs("eco_mode_charge", 0, "Eco Mode Charge"),
-        EcoModeEs("eco_mode_discharge", 6, "Eco Mode Discharge"),
+    __all_settings: Tuple[Sensor, ...] = (
         Integer("backup_supply", 12, "Backup Supply"),
         Integer("off-grid_charge", 14, "Off-grid Charge"),
         Integer("shadow_scan", 16, "Shadow Scan"),
@@ -142,18 +146,60 @@ class ES(Inverter):
         Integer("work_mode", 66, "Work Mode"),
         Integer("grid_quality_check", 68, "Grid Quality Check"),
 
-        # emulated settings for API compatibility with ET inverters
-        EcoMode("eco_mode_1", 0, "Eco Mode Power Group 1"),
+        EcoModeV1("eco_mode_1", 1793, "Eco Mode Power Group 1"),  # 0x701
+        # Byte("eco_mode_1_switch", 1796, "Eco Mode Power Group 1 Switch", "", Kind.BAT),
+        EcoModeV1("eco_mode_2", 1797, "Eco Mode Power Group 2"),
+        # Byte("eco_mode_2_switch", 1800, "Eco Mode Power Group 2 Switch", "", Kind.BAT),
+        EcoModeV1("eco_mode_3", 1801, "Eco Mode Power Group 3"),
+        # Byte("eco_mode_3_switch", 1804, "Eco Mode Power Group 3 Switch", "", Kind.BAT),
+        EcoModeV1("eco_mode_4", 1805, "Eco Mode Power Group 4"),
+        # Byte("eco_mode_4_switch", 1808, "Eco Mode Power Group 4 Switch", "", Kind.BAT),
     )
 
+    # Settings added in ARM firmware 14
+    __settings_arm_fw_14: Tuple[Sensor, ...] = (
+        EcoModeV2("eco_mode_1", 47547, "Eco Mode Power Group 1"),
+        EcoModeV2("eco_mode_2", 47553, "Eco Mode Power Group 2"),
+        EcoModeV2("eco_mode_3", 47559, "Eco Mode Power Group 3"),
+        EcoModeV2("eco_mode_4", 47565, "Eco Mode Power Group 4"),
+    )
+
+    def __init__(self, host: str, comm_addr: int = 0, timeout: int = 1, retries: int = 3):
+        super().__init__(host, comm_addr, timeout, retries)
+        if not self.comm_addr:
+            # Set the default inverter address
+            self.comm_addr = 0xf7
+        self._settings: dict[str, Sensor] = {s.id_: s for s in self.__all_settings}
+
+    def _supports_eco_mode_v2(self) -> bool:
+        if self.arm_version < 14:
+            return False
+        if "EMU" in self.serial_number:
+            return self.dsp1_version >= 11
+        if "ESU" in self.serial_number:
+            return self.dsp1_version >= 22
+        if "BPS" in self.serial_number:
+            return self.dsp1_version >= 10
+        return False
+
     async def read_device_info(self):
         response = await self._read_from_socket(self._READ_DEVICE_VERSION_INFO)
-        self.arm_firmware = response[7:12].decode("ascii").rstrip()
+        self.firmware = response[7:12].decode("ascii").rstrip()
         self.model_name = response[12:22].decode("ascii").rstrip()
         self.serial_number = response[38:54].decode("ascii")
         self.software_version = response[58:70].decode("ascii")
-        if len(self.arm_firmware) >= 5:
-            self.arm_version = int(self.arm_firmware[4], base=36)
+        try:
+            if len(self.firmware) >= 2:
+                self.dsp1_version = int(self.firmware[0:2])
+            if len(self.firmware) >= 4:
+                self.dsp2_version = int(self.firmware[2:4])
+            if len(self.firmware) >= 5:
+                self.arm_version = int(self.firmware[4], base=36)
+        except ValueError:
+            logger.exception("Error decoding firmware version %s.", self.firmware)
+
+        if self._supports_eco_mode_v2():
+            self._settings.update({s.id_: s for s in self.__settings_arm_fw_14})
 
     async def read_runtime_data(self, include_unknown_sensors: bool = False) -> Dict[str, Any]:
         raw_data = await self._read_from_socket(self._READ_DEVICE_RUNNING_DATA)
@@ -163,17 +209,21 @@ class ES(Inverter):
     async def read_setting(self, setting_id: str) -> Any:
         if setting_id == 'time':
             # Fake setting, just to enable write_setting to work (if checked as pair in read as in HA)
-            # There does not seem to be time setting/sensor evailable (or is not known)
+            # There does not seem to be time setting/sensor available (or is not known)
             return datetime.now()
-        elif setting_id == 'eco_mode_1':
-            # Emulated ET like setting
-            all_settings = await self.read_settings_data()
-            eco_charge = all_settings.get('eco_mode_charge')
-            eco_discharge = all_settings.get('eco_mode_discharge')
-            if eco_charge.is_eco_charge_mode():
-                return eco_charge.asEcoMode(True)
+        elif setting_id in ('eco_mode_1', 'eco_mode_2', 'eco_mode_3', 'eco_mode_4'):
+            setting: Sensor | None = self._settings.get(setting_id)
+            if not setting:
+                raise ValueError(f'Unknown setting "{setting_id}"')
+            count = (setting.size_ + (setting.size_ % 2)) // 2
+            if self._is_modbus_setting(setting):
+                raw_data = await self._read_from_socket(ModbusReadCommand(self.comm_addr, setting.offset, count))
+                with io.BytesIO(raw_data[5:-2]) as buffer:
+                    return setting.read_value(buffer)
             else:
-                return eco_discharge.asEcoMode(False)
+                raw_data = await self._read_from_socket(Aa55ReadCommand(setting.offset, count))
+                with io.BytesIO(raw_data[7:-2]) as buffer:
+                    return setting.read_value(buffer)
         else:
             all_settings = await self.read_settings_data()
             return all_settings.get(setting_id)
@@ -184,11 +234,25 @@ class ES(Inverter):
                 Aa55ProtocolCommand("030206" + Timestamp("time", 0, "").encode_value(value).hex(), "0382")
             )
         else:
-            raise InverterError("Operation not supported")
+            setting: Sensor | None = self._settings.get(setting_id)
+            if not setting:
+                raise ValueError(f'Unknown setting "{setting_id}"')
+            raw_value = setting.encode_value(value)
+            if len(raw_value) <= 2:
+                value = int.from_bytes(raw_value, byteorder="big", signed=True)
+                if self._is_modbus_setting(setting):
+                    await self._read_from_socket(ModbusWriteCommand(self.comm_addr, setting.offset, value))
+                else:
+                    await self._read_from_socket(Aa55WriteCommand(setting.offset, value))
+            else:
+                if self._is_modbus_setting(setting):
+                    await self._read_from_socket(ModbusWriteMultiCommand(self.comm_addr, setting.offset, raw_value))
+                else:
+                    await self._read_from_socket(Aa55WriteMultiCommand(setting.offset, raw_value))
 
     async def read_settings_data(self) -> Dict[str, Any]:
         raw_data = await self._read_from_socket(self._READ_DEVICE_SETTINGS_DATA)
-        data = self._map_response(raw_data[7:-2], self.__settings)
+        data = self._map_response(raw_data[7:-2], self.settings())
         return data
 
     async def get_grid_export_limit(self) -> int:
@@ -212,17 +276,16 @@ class ES(Inverter):
         mode = OperationMode(await self.read_setting('work_mode'))
         if OperationMode.ECO != mode:
             return mode
-        all_settings = await self.read_settings_data()
-        eco_charge = all_settings.get('eco_mode_charge')
-        eco_discharge = all_settings.get('eco_mode_discharge')
-        if eco_charge.is_eco_charge_mode():
+        ecomode = await self.read_setting('eco_mode_1')
+        if ecomode.is_eco_charge_mode():
             return OperationMode.ECO_CHARGE
-        elif eco_discharge.is_eco_discharge_mode():
+        elif ecomode.is_eco_discharge_mode():
             return OperationMode.ECO_DISCHARGE
         else:
             return OperationMode.ECO
 
-    async def set_operation_mode(self, operation_mode: OperationMode, eco_mode_power: int = 100) -> None:
+    async def set_operation_mode(self, operation_mode: OperationMode, eco_mode_power: int = 100,
+                                 eco_mode_soc: int = 100) -> None:
         if operation_mode == OperationMode.GENERAL:
             await self._set_general_mode()
         elif operation_mode == OperationMode.OFF_GRID:
@@ -230,22 +293,30 @@ class ES(Inverter):
         elif operation_mode == OperationMode.BACKUP:
             await self._set_backup_mode()
         elif operation_mode == OperationMode.ECO:
-            await self._set_eco_mode(eco_mode_power)
+            await self._set_eco_mode()
         elif operation_mode == OperationMode.PEAK_SHAVING:
             raise InverterError("Operation not supported.")
-        elif operation_mode == OperationMode.ECO_CHARGE:
-            await self._set_eco_mode(-1 * abs(eco_mode_power))
-        elif operation_mode == OperationMode.ECO_DISCHARGE:
-            await self._set_eco_mode(abs(eco_mode_power))
+        elif operation_mode in (OperationMode.ECO_CHARGE, OperationMode.ECO_DISCHARGE):
+            if eco_mode_power < 0 or eco_mode_power > 100:
+                raise ValueError()
+            if eco_mode_soc < 0 or eco_mode_soc > 100:
+                raise ValueError()
+            eco_mode: EcoMode = self._convert_eco_mode(EcoModeV2("", 0, ""))
+            if operation_mode == OperationMode.ECO_CHARGE:
+                await self.write_setting('eco_mode_1', eco_mode.encode_charge(eco_mode_power, eco_mode_soc))
+            else:
+                await self.write_setting('eco_mode_1', eco_mode.encode_discharge(eco_mode_power))
+            await self.write_setting('eco_mode_2', eco_mode.encode_off())
+            await self.write_setting('eco_mode_3', eco_mode.encode_off())
+            await self.write_setting('eco_mode_4', eco_mode.encode_off())
+            await self._set_eco_mode()
 
     async def get_ongrid_battery_dod(self) -> int:
         return await self.read_setting('dod')
 
     async def set_ongrid_battery_dod(self, dod: int) -> None:
         if 0 <= dod <= 89:
-            await self._read_from_socket(
-                Aa55ProtocolCommand("023905056001" + "{:04x}".format(100 - dod), "02b9")
-            )
+            await self._read_from_socket(Aa55WriteCommand(0x560, 100 - dod))
 
     async def _reset_inverter(self) -> None:
         await self._read_from_socket(Aa55ProtocolCommand("031d00", "039d"))
@@ -254,11 +325,11 @@ class ES(Inverter):
         return self.__sensors
 
     def settings(self) -> Tuple[Sensor, ...]:
-        return self.__settings
+        return tuple(self._settings.values())
 
     async def _set_general_mode(self) -> None:
         if self.arm_version >= 7:
-            if self._supports_new_eco_mode():
+            if self._supports_eco_mode_v2():
                 await self._clear_battery_mode_param()
             else:
                 await self._set_limit_power_for_charge(0, 0, 0, 0, 0)
@@ -283,7 +354,7 @@ class ES(Inverter):
 
     async def _set_backup_mode(self) -> None:
         if self.arm_version >= 7:
-            if self._supports_new_eco_mode():
+            if self._supports_eco_mode_v2():
                 await self._clear_battery_mode_param()
             else:
                 await self._clear_battery_mode_param()
@@ -294,14 +365,12 @@ class ES(Inverter):
         await self._set_offgrid_work_mode(0)
         await self._set_work_mode(2)
 
-    async def _set_eco_mode(self, eco_mode_power: int) -> None:
-        await self._set_limit_power_for_charge(0, 0, 23, 59, abs(eco_mode_power) if eco_mode_power < 0 else 0)
-        await self._set_limit_power_for_discharge(0, 0, 23, 59, eco_mode_power if eco_mode_power > 0 else 0)
+    async def _set_eco_mode(self) -> None:
         await self._set_offgrid_work_mode(0)
         await self._set_work_mode(3)
 
     async def _clear_battery_mode_param(self) -> None:
-        await self._read_from_socket(Aa55ProtocolCommand("0239050700010001", "02B9"))
+        await self._read_from_socket(Aa55WriteCommand(0x0700, 1))
 
     async def _set_limit_power_for_charge(self, startH: int, startM: int, stopH: int, stopM: int, limit: int) -> None:
         if limit < 0 or limit > 100:
@@ -323,20 +392,6 @@ class ES(Inverter):
     async def _set_offgrid_work_mode(self, mode: int) -> None:
         await self._read_from_socket(Aa55ProtocolCommand("033601" + "{:02x}".format(mode), "03B6"))
 
-    def _supports_new_eco_mode(self) -> bool:
-        if self.arm_version < 14:
-            return False
-        if len(self.arm_firmware) < 2:
-            return False
-        fw_version = int(self.arm_firmware[0:2])
-        if "EMU" in self.serial_number:
-            return fw_version >= 11
-        if "ESU" in self.serial_number:
-            return fw_version >= 22
-        if "BPS" in self.serial_number:
-            return fw_version >= 10
-        return False
-
     async def _set_relay_control(self, mode: int) -> None:
         param = 0
         if mode == 2:
@@ -359,3 +414,14 @@ class ES(Inverter):
 
     async def _set_work_mode(self, mode: int) -> None:
         await self._read_from_socket(Aa55ProtocolCommand("035901" + "{:02x}".format(mode), "03D9"))
+
+    def _convert_eco_mode(self, sensor: Sensor) -> Sensor | EcoMode:
+        if EcoModeV1 == type(sensor) and self._supports_eco_mode_v2():
+            return cast(EcoModeV1, sensor).as_eco_mode_v2()
+        elif EcoModeV2 == type(sensor) and not self._supports_eco_mode_v2():
+            return cast(EcoModeV2, sensor).as_eco_mode_v1()
+        else:
+            return sensor
+
+    def _is_modbus_setting(self, sensor: Sensor) -> bool:
+        return EcoModeV2 == type(sensor)
