@@ -1,18 +1,20 @@
+import os
+
 from datetime import datetime
 from unittest import TestCase, mock
 
-from goodwe.processor import GoodWeXSProcessor, InvalidDataException
+from goodwe.xs import GoodWeXSProcessor
+
+root_dir = os.path.dirname(os.path.abspath(__file__))
 
 
 class TestProcessor(TestCase):
     def test_process_data(self):
-        validator = mock.Mock()
-        validator.return_value = True
 
-        with open('sample/inverter_data', 'rb') as f:
+        with open(root_dir + '/sample/inverter_data', 'rb') as f:
             mock_data = f.read()
 
-        processor = GoodWeXSProcessor(validator)
+        processor = GoodWeXSProcessor()
         data = processor.process_data(mock_data)
 
         self.assertEqual(data.date, datetime(year=2021, month=8, day=8, hour=10, minute=49, second=52))
@@ -28,10 +30,3 @@ class TestProcessor(TestCase):
         self.assertEqual(data.temperature, 30.3)
         self.assertEqual(data.power, 189)
         self.assertEqual(data.status, 'Normal')
-
-    def test_process_data_invalid_data(self):
-        validator = mock.Mock()
-        validator.return_value = False
-
-        processor = GoodWeXSProcessor(validator)
-        self.assertRaises(InvalidDataException, processor.process_data, b'')
